 

class BloomVars:

    def __init__(self):
            
        self.pantone_colors = [
        "Living Coral",
        "Classic Blue",
        "Greenery",
        "Rose Quartz",
        "Serenity",
        "Marsala",
        "Radiant Orchid",
        "Emerald",
        "Tangerine Tango",
        "Honeysuckle",
        "Turquoise",
        "Mimosa",
        "Blue Iris",
        "Chili Pepper",
        "Sand Dollar",
        "Blue Turquoise",
        "Tigerlily",
        "Aqua Sky",
        "True Red",
        "Fuchsia Rose",
        "Cerulean",
        "Amethyst Orchid",
        "Ultra Violet",
        "Palace Blue",
        "Lime Punch",
        "Pink Lavender",
        "Crocus Petal",
        "Spring Crocus",
        "Meadowlark",
        "Cherry Tomato",
        "Chili Oil",
        "Blooming Dahlia",
        "Little Boy Blue",
        "Arcadia",
        "Ultra Violet",
        "Emperador",
        "Almost Mauve",
        "Spring Crocus",
        "Sailor Blue",
        "Harbor Mist",
        "Warm Sand",
        "Coconut Milk",
        "Soybean",
        "Mellow Yellow",
        "Fiery Red",
        "Biscay Green",
        "Coral Pink",
        "Grape Compote",
        "Storm Gray",
        "Illuminating",
        ]

        self.marine_invertebrates = [
            "Aplysia",
            "Asterias",
            "Aurelia",
            "Balanus",
            "Branchiostoma",
            "Ciona",
            "Daphnia",
            "Dugesia",
            "Echinaster",
            "Eisenia",
            "Fasciola",
            "Gammarus",
            "Glycera",
            "Haliclystus",
            "Hydra",
            "Littorina",
            "Loligo",
            "Metridium",
            "Mytilus",
            "Nereis",
            "Obelia",
            "Octopus",
            "Patella",
            "Perophora",
            "Physalia",
            "Pleurobrachia",
            "Polyclinum",
            "Porphyra",
            "Sertularia",
            "Spongia",
            "Squilla",
            "Styela",
            "Synapta",
            "Terebella",
            "Tethya",
            "Thalassiosira",
            "Tubifex",
            "Turbo",
            "Urticina",
            "Velella",
            "Vorticella",
            "Watasenia",
            "Xenoturbella",
            "Yoldia",
            "Zostera",
            "Zygocotyle",
            "Zoanthus",
            "Zoothamnium",
            "Zooxanthella",
            "Zygnema",
        ] 
