@import url('https://fonts.googleapis.com/css2?family=Dosis&display=swap');

/* CSS Variables for Easy Theme Management */
:root {
    --primary-color: #6db3f2; /* Light blue */
    --primary-hover-color: #5d9ec9; /* Slightly darker blue */
    --button-bg-color: #6db3f2;
    --button-border-color: #1003f2;
    --secondary-color: #f6b26b; /* Soft orange */
    --secondary-hover-color: #f4a460; /* Slightly darker orange */
    --accent-color: #a2d5c6; /* Mint green */
    --url-color: #6db3f2; /* Light blue */
    --addl-button-color: #6db3f2; /* Light blue */
    --background-color: #f5f5f5; /* Light grey */
    --container-bg-color: #ffffff; /* White */
    --panel-bg-color: #e6e6e6; /* Very light grey */
    --text-color: #333333; /* Dark grey (for readability) */
    --font-family: 'Dosis', sans-serif;
    --border-radius: 4px;
    --font-size-standard: 16px;
    --font-size-medium: 20px;
    --font-size-large: 24px;
    --font-size-xlarge: 36px;
    --font-size-small: 14px;
    --font-size-xsmall: 9px;
    --transition-speed: 0.3s;
    --form-in-bg-color: #e6e6e6; /* Light grey */
    --form-border-color: #cccccc; /* Soft grey */
    --form-color: black;
    --accordion-after-color: #777777; /* Medium grey */
    --status-button-color: #d6f3f8; /* Soft blue */
    --property-button-color: #898989; /* Medium grey */
    --form-container-input-color: #cccccc; /* Soft grey */
    --info-box-border-color: #b2d8d8; /* Soft teal */
    --hilight-color: #f9de75; /* Light yellow */
    --a: #6db3f2; /* Light blue */
    --a-visited: #b4a7d6; /* Soft purple */
    --a-hover: #f4cccc; /* Soft pink */
    --a-active: #c9daf8; /* Soft blue */
    --close-dialog-btn-hover-color: #b6d7a8; /* Soft green */
    --table-border-color: #d9d2e9; /* Very light purple */
    --th-background-color: #ead1dc; /* Soft pink */
    --filter-button-active: #93c47d; /* Soft green */
    --filter-button-color: black;
    --workflow-non-step-bg-color: #9fc5e8; /* Soft blue */
    --accordion-prefix: #d0e0e3; /* Very light teal */
    --disabled-action-btn: #d9d9d9; /* Light grey */
}

