@import url('https://fonts.googleapis.com/css2?family=Dosis&display=swap');

/* CSS Variables for Easy Theme Management */
:root {
    --primary-color: #ff3856; /* updated to the vivid red */
    --primary-hover-color: #61fe39; /* updated to a bright green for contrast */
    --button-bg-color: #ff3856;
    --button-border-color: #61fe39;
    --secondary-color: #3a47ff; /* updated to the strong blue */
    --secondary-hover-color: #36525c; /* updated to the deep blue-gray */
    --accent-color:#151bb1;/* updated to the soft violet */
    --url-color: #3a47ff; /* updated to match the strong blue */
    --addl-button-color: #3a47ff; /* updated to match the strong blue */
    --background-color: #36525c; /* updated to the deep blue-gray */
    --container-bg-color: #36525c; /* updated to match background color */
    --panel-bg-color: #36525c; /* slightly darker shade for depth */
    --text-color: #ffffff; /* kept white for readability */
    --font-family: 'Dosis', sans-serif; /* unchanged */
    --border-radius: 4px; /* unchanged */
    --font-size-standard: 16px; /* unchanged */
    --font-size-medium: 20px; /* unchanged */
    --font-size-large: 24px; /* unchanged */
    --font-size-xlarge: 36px; /* unchanged */
    --font-size-small: 14px; /* unchanged */
    --font-size-xsmall: 9px; /* unchanged */
    --transition-speed: 0.3s; /* unchanged */
    --form-in-bg-color: #8c96ee; /* updated to the soft violet */
    --form-border-color: #36525c; /* updated to the deep blue-gray */
    --form-color: #ffffff; /* changed to white for readability */
    --accordion-after-color: #61fe39; /* updated to the bright green */
    --status-button-color: #61fe39; /* updated to the bright green */
    --property-button-color: #ff3856; /* updated to the vivid red */
    --form-container-input-color: #36525c; /* updated to the deep blue-gray */
    --info-box-border-color: #3a47ff; /* updated to the strong blue */
    --hilight-color: rgb(182, 232, 234); /* unchanged */
    --a: #3a47ff; /* updated to match the strong blue */
    --a-visited: #8c96ee; /* updated to the soft violet */
    --a-hover: #ff3856; /* updated to the vivid red */
    --a-active: #61fe39; /* updated to the bright green */
    --close-dialog-btn-hover-color: #ff3856; /* updated to the vivid red */
    --table-border-color: #36525c; /* updated to the deep blue-gray */
    --th-background-color: #8c96ee; /* updated to the soft violet */
    --filter-button-active: #3a47ff; /* updated to the strong blue */
    --filter-button-color: white; /* unchanged */
    --workflow-non-step-bg-color: rgb(82,11,149); /* updated to the deep blue-gray */
    --accordion-prefix: #8c96ee; /* updated to the soft violet */
    --disabled-action-btn: #acb6b7; /* unchanged */
}

